﻿-- Author      : Hentaya@Elune
-- Create Date : 11/22/2012  7:24:22 PM
-- Modify Date : 11/28/2012 12:48:13 AM
-- Modify Date : 11/17/2015 12:27:18 PM - Initial 11th anniversary mods; minimal testing
-- Modify Date : 11/26/2017 04:03:00 AM - remove button show conditions for combat (Item can now be used in combat)
-- Modify Date : 11/26/2017 06:37:00 PM - rework button show conditions for live/dead

local name = "Celebrate!"
Celebrate = LibStub("AceAddon-3.0"):NewAddon(name)

local isLoginUse = true
local version = GetAddOnMetadata(name, "Version"):match( "^([%d.]+)" )
local useButton 

local CELEBRATE_AURA_NAME = "WoW's 13th Anniversary"
local CELEBRATE_ITEM_NAME = "Celebration Package"
local CELEBRATE_TEXTURE_PATH = "INTERFACE\\ICONS\\TEMP"
local EVENT_PLAYER_ALIVE = "PLAYER_ALIVE"
local EVENT_BUFFS_CHANGED = "UNIT_AURA"
local EVENT_CB_ON = "CB_ON"
local EVENT_CB_OFF = "CB_OFF"
local EVENT_CB_ENABLED = "CB_ENABLED"
local UNIT_PLAYER = "player"

function Celebrate:OnInitialize()
	if CelebrateDB == nil or CelebrateDB == "" then
		DEFAULT_CHAT_FRAME:AddMessage("|c30603090[" .. name .. "]|r: Initializing DB on first use")

		CelebrateDB = {
			Version = version;
			UseItem = false;
		}
	elseif  CelebrateDB.Version ~= version then
		CelebrateDB.Version = version 
	end		

	SlashCmdList["CELEBRATE"] =  Celebrate_SlashCmdHandler
	SLASH_CELEBRATE1 = "/cb"
	SLASH_CELEBRATE2 = "/celebrate!"
	SLASH_CELEBRATE3 = "/celebrate"	

	DEFAULT_CHAT_FRAME:AddMessage("|c30603090[" .. name .. "]|r v" .. version  .. " loaded and ".. Celebrate_Status() ..". Use |cFF00FFFF/cb help|r for a list of options.")	
end

function Celebrate:OnEnable()
	if useButton == nil then
		useButton = Celebrate_CreateUseButton()
	end

	useButton:RegisterEvent(EVENT_PLAYER_ALIVE)
	useButton:RegisterEvent(EVENT_BUFFS_CHANGED)
	useButton:SetScript("OnEvent",Celebrate_HideOrShow)

	Celebrate_HideOrShow(self,EVENT_CB_ENABLED)
end

function Celebrate_CreateUseButton()
	local _button = CreateFrame("BUTTON", nil, UIParent, "SecureActionButtonTemplate")
	local _texture = _button:CreateTexture("useButtonTexture")

	_button:SetAttribute("type", "item");
	_button:SetAttribute("item", CELEBRATE_ITEM_NAME);
	_button:RegisterForClicks("LeftButtonUp")
	_button:SetScript("PostClick", Celebrate_HideUseButton)

	_button:SetAlpha(1)
	_button:SetWidth(690)
	_button:SetHeight(690)
	_button:SetPoint("CENTER", UIParent, "CENTER")

	_texture:SetTexture(CELEBRATE_TEXTURE_PATH)
	_texture:SetBlendMode("disable")
	_texture:SetWidth(_button:GetWidth())
	_texture:SetHeight(_button:GetHeight())
	_texture:SetPoint("CENTER", UIParent, "CENTER")

	_button:SetNormalTexture(_texture)
	_button:SetPushedTexture(_texture)
	_button:SetHighlightTexture(_texture)
	_button:SetDisabledTexture(_texture)

	return _button
end

function Celebrate_HideOrShow(self, event)
	local _active = Celebrate_IsActive()

	--DEFAULT_CHAT_FRAME:AddMessage ("IsActive? " .. tostring(_active));
	--DEFAULT_CHAT_FRAME:AddMessage ("---EVENT2: " .. event);
	if not _active and not UnitIsDeadOrGhost(UNIT_PLAYER) and not IsFalling() and event ~= EVENT_CB_OFF then
		Celebrate_ShowUseButton()
	else
		Celebrate_HideUseButton()
	end
end

function Celebrate_HideUseButton()	
	if useButton:GetNormalTexture() ~= nil then
		useButton:GetNormalTexture():Hide()
		useButton:GetPushedTexture():Hide()
		useButton:GetHighlightTexture():Hide()
		useButton:GetDisabledTexture():Hide()
	end

	useButton:Hide()
end

function Celebrate_ShowUseButton()
	local canUse, cantUseReason = Celebrate_CanUse()

	if canUse then
		if  CelebrateDB.UseItem and not Celebrate.IsActive then
			useButton:Show()		

			if useButton:GetNormalTexture() ~= nil then
				useButton:GetNormalTexture():Show()
				useButton:GetPushedTexture():Show()
				useButton:GetHighlightTexture():Show()
				useButton:GetDisabledTexture():Show()
			end
		end
	else
		if cantUseReason ~=  "You have no " .. CELEBRATE_ITEM_NAME .. "." or isLoginUse then
			DEFAULT_CHAT_FRAME:AddMessage ("Cannot use Celebrate!: ".. cantUseReason);
			isLoginUse = false
		end
	end
end

function Celebrate_SlashCmdHandler(cmd)
    if cmd == "help" then
		DEFAULT_CHAT_FRAME:AddMessage ("/cb USAGE:");
		DEFAULT_CHAT_FRAME:AddMessage ("    |cFF00FFFF/cb    |r: report status of |c30603090[" .. name .. "]|r");
		DEFAULT_CHAT_FRAME:AddMessage ("    |cFF00FFFF/cb on |r: activate prompt for use of " .. CELEBRATE_ITEM_NAME .. " when expired");
		DEFAULT_CHAT_FRAME:AddMessage ("    |cFF00FFFF/cb off|r: deactivate prompting.");
	else
		if cmd == "on" then
			CelebrateDB.UseItem = true
			Celebrate_HideOrShow(self,EVENT_CB_ON)
		elseif cmd == "off" then
			CelebrateDB.UseItem = false;
			Celebrate_HideOrShow(self,EVENT_CB_OFF)
		end
		

		DEFAULT_CHAT_FRAME:AddMessage ("|c30603090Celebrate!|r is " .. Celebrate_Status());
		if cmd ~= "on" and cmd ~= "off" then
			DEFAULT_CHAT_FRAME:AddMessage ("   (|cFF00FFFF/cb help|r for usage).");
		end
	end
end

function Celebrate_Status()
	if CelebrateDB.UseItem then
		return "on"
	else
		return "off"
	end
end

function Celebrate_CanUse()
	local _canUse = true
	local _cantUseReason = ""

	local itemCount = GetItemCount(CELEBRATE_ITEM_NAME,true)
	local itemCountInInventory = GetItemCount(CELEBRATE_ITEM_NAME) 

	if (itemCount == 0) then
		_canUse = false
		_cantUseReason = "You have no " .. CELEBRATE_ITEM_NAME .. "."
		CelebrateDB.UseItem = false;
	elseif (itemCountInInventory == 0) then
		_canUse = false
		_cantUseReason = CELEBRATE_ITEM_NAME .. " is in the bank."
	end

	return _canUse, _cantUseReason
end

function Celebrate_IsActive()
	local _isActive = false
	
	_isActive = UnitAura(UNIT_PLAYER, CELEBRATE_AURA_NAME) == CELEBRATE_AURA_NAME

	return _isActive
end
